import { system } from "@minecraft/server";
export function sleep(ticks) {
    return new Promise((resolve) => system.runTimeout(resolve, ticks));
}
export function getOrientation(playerRotationY) {
    switch (true) {
        case playerRotationY > 70 && playerRotationY < 170:
            return 4;
        case playerRotationY > -45 && playerRotationY < 65:
            return 1;
        case playerRotationY > -100 && playerRotationY < -10:
            return 3;
        case playerRotationY >= 148 || playerRotationY < -100:
            return 2;
        default:
            return 0;
    }
}
export function getOrientationX(playerRotationX) {
    switch (true) {
        case playerRotationX > 20 && playerRotationX < -20:
            return -1;
        case playerRotationX > 37:
            return 1;
        case playerRotationX < -21:
            return 0;
        default:
            return -1;
    }
}
export async function getNearbyBlocks(pos, dimension, cb) {
    for (let x = pos.x - 1; x <= pos.x + 1; x++) {
        for (let y = pos.y - 1; y <= pos.y + 1; y++) {
            for (let z = pos.z - 1; z <= pos.z + 1; z++) {
                try {
                    const block = dimension.getBlock({ x, y, z });
                    if (block)
                        cb(block, dimension, pos);
                }
                catch (e) { }
            }
        }
    }
}
export function detectItemOnInventory(player, itemTypeId) {
    const inv = player.getComponent('inventory');
    const container = inv.container;
    for (let i = 0; i < container.size; i++) {
        const item = container.getItem(i);
        if (!item)
            continue;
        if (item.typeId == itemTypeId)
            return { item, slot: i };
    }
}
export function hasTag(itemStack, tag, tagRegexp = null) {
    return itemStack
        .getTags()
        .find(itemTag => tagRegexp ? tagRegexp.test(itemTag) : itemTag == tag);
}
export function getRandomItem(array) {
    if (!array || array.length === 0)
        return;
    const randomIndex = Math.floor(Math.random() * array.length);
    return array[randomIndex];
}
export class Vector {
    static up = { x: 0, y: 1, z: 0 };
    static down = { x: 0, y: -1, z: 0 };
    static forward = { x: 0, y: 0, z: 1 };
    static back = { x: 0, y: 0, z: -1 };
    static left = { x: -1, y: 0, z: 0 };
    static right = { x: 1, y: 0, z: 0 };
    static zero = { x: 0, y: 0, z: 0 };
    static multiply(vectorA, value) {
        return {
            x: vectorA.x * (typeof value === 'number' ? value : value.x ?? 1),
            y: vectorA.y * (typeof value === 'number' ? value : value.y ?? 1),
            z: vectorA.z * (typeof value === 'number' ? value : value.z ?? 1)
        };
    }
    static add(vectorA, vectorB) {
        return {
            x: vectorA.x + (vectorB.x ?? 0),
            y: vectorA.y + (vectorB.y ?? 0),
            z: vectorA.z + (vectorB.z ?? 0)
        };
    }
    static subtract(vectorA, vectorB) {
        return {
            x: vectorA.x - (vectorB.x ?? 0),
            y: vectorA.y - (vectorB.y ?? 0),
            z: vectorA.z - (vectorB.z ?? 0)
        };
    }
    static distance(vectorA, vectorB) {
        return Math.sqrt(Math.pow(vectorA.x - vectorB.x, 2)
            + Math.pow(vectorA.y - vectorB.y, 2)
            + Math.pow(vectorA.z - vectorB.z, 2));
    }
}
